@@ -5,17 +5,19 @@ import json
 import os
 from copy import deepcopy
 from functools import wraps
+from io import StringIO
 from typing import Optional, List, Dict, Union, Any, Type, Callable, Tuple
 
 from flask import Flask, Blueprint, render_template, request, make_response, current_app
-from pydantic import ValidationError, BaseModel
+from pydantic import ValidationError, BaseModel, AnyUrl
 
+from .markdown import openapi_to_markdown
 from .models import Info, APISpec, Tag, Components
-from .models.common import Reference
+from .models.common import Reference, ExternalDocumentation
 from .models.security import SecurityScheme
 from .plugins import OAuthConfig
 from .utils import _parse_rule, get_operation, get_responses, parse_and_store_tags, parse_parameters, \
-    validate_responses, parse_method, validate_response
+    validate_responses_type, parse_method, validate_response
 
 
 def _do_wrapper(
@@ -30,16 +32,16 @@ def _do_wrapper(
         **kwargs: Any
 ) -> Any:
     """
-    validate request and response
+    Validate requests and responses
     :param func: view func
     :param responses: response model
-    :param header:
-    :param cookie:
-    :param path:
-    :param query:
-    :param form:
-    :param body:
-    :param kwargs: path args
+    :param header: header model
+    :param cookie: cookie model
+    :param path: path model
+    :param query: query model
+    :param form: form model
+    :param body: body model
+    :param kwargs: path parameters
     :return:
     """
     # validate header, cookie, path and query
@@ -126,7 +128,7 @@ class APIBlueprint(Blueprint):
         Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel], Dict[
             str, Type[BaseModel]]]:
         """
-        collect openapi spec information
+        Collect openapi specification information
         :param rule: flask route
         :param func: flask view_func
         :param tags: api tag
@@ -139,8 +141,8 @@ class APIBlueprint(Blueprint):
         if self.doc_ui is True and doc_ui is True:
             if responses is None:
                 responses = {}
-            validate_responses(responses)
-            validate_responses(self.abp_responses)
+            validate_responses_type(responses)
+            validate_responses_type(self.abp_responses)
             # global response combine api responses
             combine_responses = deepcopy(self.abp_responses)
             combine_responses.update(**responses)
@@ -302,7 +304,7 @@ class OpenAPI(Flask):
                  **kwargs: Any
                  ) -> None:
         """
-        Based Flask, provide REST api, swagger-ui and redoc.
+        Based on Flask. Provide REST api, swagger-ui and redoc.
         :param import_name: just flask import_name
         :param info: see https://spec.openapis.org/oas/v3.0.3#info-object
         :param securitySchemes: see https://spec.openapis.org/oas/v3.0.3#security-scheme-object
@@ -336,7 +338,7 @@ class OpenAPI(Flask):
 
     def init_doc(self) -> None:
         """
-        provide swagger-ui and redoc
+        Provide swagger-ui and redoc
         :return:
         """
         _here = os.path.dirname(__file__)
@@ -369,6 +371,11 @@ class OpenAPI(Flask):
                 oauth_config=self.oauth_config.dict() if self.oauth_config else None
             )
         )
+        blueprint.add_url_rule(
+            rule='/markdown',
+            endpoint='markdown',
+            view_func=lambda: self.export_to_markdown()
+        )
         blueprint.add_url_rule(
             rule='/',
             endpoint='index',
@@ -376,10 +383,32 @@ class OpenAPI(Flask):
         )
         self.register_blueprint(blueprint)
 
+    def export_to_markdown(self):
+        """Experimental"""
+        md = StringIO()
+
+        md.write(openapi_to_markdown(self.api_doc))
+
+        r = make_response(md.getvalue())
+        r.headers['Content-Disposition'] = 'attachment; filename=api.md'
+
+        return r
+
     @property
     def api_doc(self) -> Dict:
-        """generate spec json"""
-        spec = APISpec(openapi=self.openapi_version, info=self.info)
+        """Generate spec json"""
+        spec = APISpec(
+            openapi=self.openapi_version,
+            info=self.info,
+            externalDocs=ExternalDocumentation(
+                url=AnyUrl(
+                    url=f'/{self.api_name}/markdown',
+                    scheme='',
+                    host=''
+                ),
+                description='Export to markdown'
+            )
+        )
         spec.tags = self.tags or None
         spec.paths = self.paths
         self.components.schemas = self.components_schemas
@@ -388,7 +417,7 @@ class OpenAPI(Flask):
         return json.loads(spec.json(by_alias=True, exclude_none=True))
 
     def register_api(self, api: APIBlueprint) -> None:
-        """register APIBlueprint"""
+        """Register APIBlueprint"""
         for tag in api.tags:
             if tag.name not in self.tag_names:
                 self.tags.append(tag)
@@ -409,7 +438,7 @@ class OpenAPI(Flask):
         Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel], Dict[
             str, Type[BaseModel]]]:
         """
-        collect openapi spec information
+        Collect openapi specification information
         :param rule: flask route
         :param func: flask view_func
         :param tags: api tag
@@ -422,8 +451,8 @@ class OpenAPI(Flask):
         if doc_ui is True:
             if responses is None:
                 responses = {}
-            validate_responses(responses)
-            validate_responses(self.responses)
+            validate_responses_type(responses)
+            validate_responses_type(self.responses)
             # global response combine api responses
             combine_responses = deepcopy(self.responses)
             combine_responses.update(**responses)
