@@ -19,7 +19,9 @@ from .status import HTTP_STATUS
 
 
 def _parse_rule(rule: str) -> str:
-    """parse route"""
+    """Flask route conversion to openapi:
+    /pet/<petId> --> /pet/{petId}
+    """
     uri = ''
     for converter, args, variable in parse_rule(str(rule)):
         if converter is None:
@@ -30,7 +32,7 @@ def _parse_rule(rule: str) -> str:
 
 
 def get_operation(func: Callable) -> Operation:
-    # get func documents
+    """Return a Operation object with summary and description."""
     doc = inspect.getdoc(func) or ''
     doc = doc.strip()
     lines = doc.split('\n')
@@ -43,23 +45,28 @@ def get_operation(func: Callable) -> Operation:
 
 
 def get_func_parameter(func: Callable, arg_name='path') -> Type[BaseModel]:
-    """get func parameters"""
+    """Get view-func parameters.
+    arg_name has six parameters to choose from: path, query, form, body, header, cookie.
+    """
     signature = inspect.signature(func)
     p = signature.parameters.get(arg_name)
     return p.annotation if p else None
 
 
 def get_schema(obj: Type[BaseModel]) -> dict:
+    """Pydantic model conversion to openapi schema"""
     assert inspect.isclass(obj) and \
            issubclass(obj, BaseModel), f"{obj} is invalid `pydantic.BaseModel`"
     return obj.schema(ref_template=OPENAPI3_REF_TEMPLATE)
 
 
-def parse_header(header: Type[BaseModel]) -> List[Parameter]:
-    """parse args(header)"""
+def parse_header(header: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
+    """Parse header model"""
     schema = get_schema(header)
     parameters = []
+    components_schemas = dict()
     properties = schema.get('properties')
+    definitions = schema.get('definitions')
 
     if properties:
         for name, value in properties.items():
@@ -71,15 +78,19 @@ def parse_header(header: Type[BaseModel]) -> List[Parameter]:
                 "schema": Schema(**value)
             }
             parameters.append(Parameter(**data))
-
-    return parameters
+    if definitions:
+        for name, value in definitions.items():
+            components_schemas[name] = Schema(**value)
+    return parameters, components_schemas
 
 
-def parse_cookie(cookie: Type[BaseModel]) -> List[Parameter]:
-    """parse args(cookie)"""
+def parse_cookie(cookie: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
+    """Parse cookie model"""
     schema = get_schema(cookie)
     parameters = []
+    components_schemas = dict()
     properties = schema.get('properties')
+    definitions = schema.get('definitions')
 
     if properties:
         for name, value in properties.items():
@@ -91,15 +102,19 @@ def parse_cookie(cookie: Type[BaseModel]) -> List[Parameter]:
                 "schema": Schema(**value)
             }
             parameters.append(Parameter(**data))
-
-    return parameters
+    if definitions:
+        for name, value in definitions.items():
+            components_schemas[name] = Schema(**value)
+    return parameters, components_schemas
 
 
-def parse_path(path: Type[BaseModel]) -> List[Parameter]:
-    """parse args(path)"""
+def parse_path(path: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
+    """Parse path model"""
     schema = get_schema(path)
     parameters = []
+    components_schemas = dict()
     properties = schema.get('properties')
+    definitions = schema.get('definitions')
 
     if properties:
         for name, value in properties.items():
@@ -111,11 +126,14 @@ def parse_path(path: Type[BaseModel]) -> List[Parameter]:
                 "schema": Schema(**value)
             }
             parameters.append(Parameter(**data))
-
-    return parameters
+    if definitions:
+        for name, value in definitions.items():
+            components_schemas[name] = Schema(**value)
+    return parameters, components_schemas
 
 
 def parse_query(query: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
+    """Parse query model"""
     schema = get_schema(query)
     parameters = []
     components_schemas = dict()
@@ -139,6 +157,7 @@ def parse_query(query: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
 
 
 def parse_form(form: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
+    """Parse form model"""
     schema = get_schema(form)
     content = None
     components_schemas = dict()
@@ -166,6 +185,7 @@ def parse_form(form: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
 
 
 def parse_body(body: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
+    """Parse body model"""
     schema = get_schema(body)
     content = None
     components_schemas = dict()
@@ -249,12 +269,12 @@ def get_responses(responses: dict, components_schemas: dict, operation: Operatio
     operation.responses = _responses
 
 
-def validate_responses(responses: Dict[str, Type[BaseModel]]) -> None:
+def validate_responses_type(responses: Dict[str, Type[BaseModel]]) -> None:
     assert isinstance(responses, dict), "invalid `dict`"
 
 
 def validate_response(resp: Any, responses: Dict[str, Type[BaseModel]]) -> None:
-    """validate response"""
+    """Validate response"""
     print("Warning: "
           "You are using `VALIDATE_RESPONSE=True`, "
           "please do not use it in the production environment, "
@@ -290,7 +310,7 @@ def parse_and_store_tags(
         old_tag_names: List[str] = None,
         operation: Operation = None
 ) -> None:
-    """store tags
+    """Store tags
     :param new_tags: api tag
     :param old_tags: openapi doc tags
     :param old_tag_names: openapi doc tag names
@@ -327,15 +347,18 @@ def parse_parameters(
     if doc_ui is False:
         return header, cookie, path, query, form, body
     if header:
-        _parameters = parse_header(header)
+        _parameters, _components_schemas = parse_header(header)
         parameters.extend(_parameters)
+        components_schemas.update(**_components_schemas)
     if cookie:
-        _parameters = parse_cookie(cookie)
+        _parameters, _components_schemas = parse_cookie(cookie)
         parameters.extend(_parameters)
+        components_schemas.update(**_components_schemas)
     if path:
         # get args from route path
-        _parameters = parse_path(path)
+        _parameters, _components_schemas = parse_path(path)
         parameters.extend(_parameters)
+        components_schemas.update(**_components_schemas)
     if query:
         # get args from route query
         _parameters, _components_schemas = parse_query(query)
